@@ -1,6 +1,7 @@
 """WYN360 Agent - AI coding assistant using pydantic_ai"""
 
 import os
+import sys
 from typing import List, Dict, Any
 from pydantic_ai import Agent, RunContext
 from pydantic_ai.models.anthropic import AnthropicModel
@@ -48,8 +49,10 @@ class WYN360Agent:
                 self.read_file,
                 self.write_file,
                 self.list_files,
-                self.get_project_info
-            ]
+                self.get_project_info,
+                self.execute_command
+            ],
+            retries=3  # Allow up to 3 retries for tool calls
         )
 
     def _get_system_prompt(self) -> str:
@@ -70,8 +73,64 @@ Guidelines:
 - For existing projects, analyze the structure before making changes
 - Be proactive in suggesting best practices and improvements
 
-When a user asks you to create files, use the write_file tool to save them.
-When analyzing projects, use list_files and read_file tools first.
+**File Operation Intelligence:**
+
+**CREATING NEW FILES** - When user says:
+- "write a .py script", "write a script", "generate a script"
+- "create new", "make another", "build a separate"
+- Specifies a new filename explicitly
+- Asks to "write code to analyze", "write code to explore"
+
+ACTION:
+- Use write_file with overwrite=False (default)
+- If write_file fails with "already exists" error, automatically retry with overwrite=True
+- Do NOT read_file first (the file doesn't exist yet)
+- Suggest a descriptive filename if user doesn't specify one
+
+EXAMPLES:
+- "write a .py script to analyze data.csv" → Create analyze_data.py with overwrite=False
+- "generate a script for data exploration" → Create data_exploration.py with overwrite=False
+- If it fails (file exists), immediately retry: write_file with overwrite=True
+
+**UPDATING EXISTING FILES** - When user says:
+- "add feature", "update", "improve", "modify", "change"
+- "add XYZ to my app", "improve the chatbot"
+- References an existing file by name
+
+ACTION:
+- ALWAYS use read_file first to understand current state
+- Then use write_file with overwrite=True to save changes
+- Include context about what changed in your response
+
+**Critical Rules:**
+1. If write_file fails with "File already exists" → Retry immediately with overwrite=True
+2. Don't give up after first write_file failure - try with overwrite=True
+3. For "write/generate script" requests → ALWAYS create new file (use overwrite=False, then True if needed)
+
+**Command Execution:**
+
+You can execute shell commands using the execute_command tool.
+
+When user says "run", "execute", "install", "start", or asks you to run a script:
+- Understand what they want to run
+- Use execute_command with the appropriate command
+- Explain what the command will do before running it
+
+Common patterns:
+- Python scripts: execute_command("python script.py")
+- UV commands:
+  - execute_command("uv init project_name")
+  - execute_command("uv add package1 package2")
+  - execute_command("uv run python script.py")
+- Streamlit apps: execute_command("uv run streamlit run app.py")
+- Shell scripts: execute_command("bash script.sh")
+- Any CLI tool: execute_command("npm install"), execute_command("docker ps"), etc.
+
+Notes:
+- The user will be asked to confirm before execution (handled automatically)
+- Commands run with user's full permissions in the current directory
+- Default timeout is 300 seconds (5 minutes), adjust if needed
+- Always provide context about expected results
 """
 
     async def read_file(self, ctx: RunContext[None], file_path: str) -> str:
@@ -150,6 +209,65 @@ When analyzing projects, use list_files and read_file tools first.
 
         return summary
 
+    async def execute_command(
+        self,
+        ctx: RunContext[None],
+        command: str,
+        timeout: int = 300
+    ) -> str:
+        """
+        Execute a shell command and return its output.
+
+        This tool can execute any shell command including:
+        - Python scripts: "python run_analysis.py"
+        - UV commands: "uv init my_project", "uv add torch scikit-learn", "uv run streamlit run app.py"
+        - Shell scripts: "bash setup.sh"
+        - Any CLI tool: "npm install", "docker run", etc.
+
+        Args:
+            command: Full command string to execute
+            timeout: Maximum execution time in seconds (default: 300)
+
+        Returns:
+            Command output or error message
+
+        Note:
+            User will be asked to confirm before execution.
+            Commands run with user's full permissions in the current directory.
+        """
+        from .utils import execute_command_safe
+
+        # Ask for user confirmation in interactive mode
+        # Skip confirmation in non-interactive mode (tests) or if disabled via env var
+        if sys.stdin.isatty() and os.getenv('WYN360_SKIP_CONFIRM') != '1':
+            # Clear any spinners and make prompt very visible
+            print("\n" + "="*70)
+            print("⚠️  COMMAND EXECUTION CONFIRMATION")
+            print("="*70)
+            print(f"Command: {command}")
+            print(f"Directory: {os.getcwd()}")
+            print("Permissions: Full user permissions")
+            print("="*70)
+            print("\n>>> WAITING FOR YOUR RESPONSE <<<\n")
+            sys.stdout.flush()  # Force output to appear immediately
+
+            response = input("Execute this command? (y/N): ").strip().lower()
+            print()  # Add spacing after response
+
+            if response not in ['y', 'yes']:
+                return "❌ Command execution cancelled by user."
+
+        success, output, return_code = execute_command_safe(command, timeout)
+
+        if success:
+            result = f"✅ Command executed successfully (exit code {return_code})\n\n"
+            result += f"Output:\n{output}"
+            return result
+        else:
+            result = f"❌ Command failed (exit code {return_code})\n\n"
+            result += f"Error output:\n{output}"
+            return result
+
     async def chat(self, user_message: str) -> str:
         """
         Process a user message and generate a response.
