@@ -7,6 +7,8 @@ from pathlib import Path
 from dotenv import load_dotenv
 from rich.console import Console
 from rich.markdown import Markdown
+from prompt_toolkit import PromptSession
+from prompt_toolkit.key_binding import KeyBindings
 from .agent import WYN360Agent
 
 console = Console()
@@ -58,8 +60,11 @@ def main(api_key, model):
     console.print()
     console.print("[yellow]Commands:[/yellow]")
     console.print("  • Type your request to chat with the assistant")
+    console.print("  • Press [bold]Enter[/bold] to submit, [bold]Ctrl+Enter[/bold] for new line")
     console.print("  • Type [bold]'exit'[/bold] or [bold]'quit'[/bold] to end the session")
     console.print()
+    console.print("[yellow]Note:[/yellow] You'll be asked to confirm before executing any commands")
+    console.print()
 
     # Initialize agent
     try:
@@ -81,10 +86,26 @@ async def chat_loop(agent: WYN360Agent):
     Args:
         agent: The WYN360Agent instance
     """
+    # Create key bindings for multi-line input
+    # Ctrl+Enter = newline, Enter = submit
+    kb = KeyBindings()
+
+    @kb.add('c-j')  # Ctrl+Enter (Ctrl+J is the terminal code for Ctrl+Enter)
+    def _(event):
+        event.current_buffer.insert_text('\n')
+
+    # Create prompt session
+    session = PromptSession(
+        multiline=False,
+        key_bindings=kb,
+        prompt_continuation=lambda width, line_number, is_soft_wrap: '... '
+    )
+
     while True:
         try:
             # Get user input
-            user_input = console.input("[bold green]You:[/bold green] ")
+            console.print("[bold green]You:[/bold green]")
+            user_input = await session.prompt_async("")
 
             # Check for exit commands
             if user_input.lower().strip() in ['exit', 'quit', 'q']:
